/*
 * TCU support for various chips.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/hw_mmio_register_block.h>

// TCU channel.

class Tcu_channel
{
protected:
  uint8_t _channel;
  Hw::Register_block<32> _regs;

  uint32_t get_field(uint32_t reg, uint32_t mask, uint8_t shift);
  void set_field(uint32_t reg, uint32_t mask, uint8_t shift, uint32_t value);

public:
  explicit Tcu_channel(l4_addr_t start, uint8_t channel);

  virtual void disable();
  virtual void enable();
  virtual bool is_enabled();
  virtual uint8_t get_clock();
  virtual void set_clock(uint8_t clock);
  virtual uint32_t get_prescale();
  virtual void set_prescale(uint32_t prescale);
  virtual uint32_t get_counter();
  virtual void set_counter(uint32_t value);
  virtual uint8_t get_count_mode();
  virtual void set_count_mode(uint8_t mode);
  virtual uint32_t get_full_data_value();
  virtual void set_full_data_value(uint32_t value);
  virtual uint32_t get_half_data_value();
  virtual void set_half_data_value(uint32_t value);
};

// TCU device control.

class Tcu_chip
{
protected:
  l4_addr_t _start, _end;

  virtual unsigned int num_channels() = 0;

  virtual Tcu_channel *_get_channel(l4_addr_t addr, uint8_t channel) = 0;

public:
  explicit Tcu_chip(l4_addr_t start, l4_addr_t end);

  Tcu_channel *get_channel(uint8_t channel);
};

#endif /* __cplusplus */
